#include "menu.h"

menuObject::menuObject(QLabel * parent):QLabel(parent, Qt::X11BypassWindowManagerHint){
	if (QX11Info::isCompositingManagerRunning()) dbusServiceRegister ("Menu", this);
	else qDebug()<<"Compositing manager not running. DBus interface \"/Menu\" disabled.";
	timeOff = new QTimer (this);
	connect (timeOff, SIGNAL(timeout()), this, SLOT(hideMenu()));

	bgPic = new QLabel ("",this);
	slPic = new QLabel ("",this);
	bgPic->setScaledContents (true);
	slPic->setScaledContents (true);

	scene = new QGraphicsScene (this);
	view = new QGraphicsView (this);

	olPic = new QLabel ("",this);
	clPic = new QLabel ("",this);
	fgPic = new QLabel ("",this);
	olPic->setScaledContents (true);
	clPic->setScaledContents (true);
	fgPic->setScaledContents (true);

	view->setFrameShape(QFrame::NoFrame);
	view->viewport()->setAutoFillBackground(false);
}

menuObject::~menuObject(){}

void menuObject::showMenu (QString profile){
	cProfile=profile;
	current=loadParameterFromConfig (profile,"MenuCurrentItem").toInt();
	setConfigValue (profile,"MenuMSize");
	setConfigValue (profile,"MenuMsTimeOff");
	setConfigValue (profile,"MenuItemPicSize");
	setConfigValue (profile,"MenuKCenter");
	setConfigValue (profile,"MenuItems");
	setConfigValue (profile,"MenuItemShadowPic");
	setConfigValue (profile,"MenuItemOverPic");
	setConfigValue (profile,"MenuBackgroundPic");
	setConfigValue (profile,"MenuForegroundPic");
	setConfigValue (profile,"MenuTextProfile");
	setConfigValue (profile,"MenuLeft");
	setConfigValue (profile,"MenuTop");
	setConfigValue (profile,"MenuOffset");
	setConfigValue (profile,"MenuOpacity");
	setConfigValue (profile,"MenuAnimationTime");
	setConfigValue (profile,"MenuAnimationTypeMove");
	setConfigValue (profile,"MenuAnimationTypeCenter");
	setConfigValue (profile,"MenuAnimationTypeFade");

	offset=strToInt (workValue["MenuOffset"],10);
	mSize=strToInt (workValue["MenuMSize"],600);
	msTimeOff=strToInt (workValue["MenuMsTimeOff"],5000);
	itemPicSize=strToInt (workValue["MenuItemPicSize"],100);
	kCenter=QString (workValue["MenuKCenter"]).toDouble();
	animTime=strToInt (workValue["MenuAnimationTime"],300);
	animTypeMove=QEasingCurve::Type (strToInt (workValue["MenuAnimationTypeMove"],19));
	animTypeCenter=QEasingCurve::Type (strToInt (workValue["MenuAnimationTypeCenter"],30));
	animTypeFade=QEasingCurve::Type (strToInt (workValue["MenuAnimationTypeFade"],19));

	QStringList mItems = workValue["MenuItems"].split (",");
	itemsCount=mItems.count();
	if (current >= itemsCount) current=0;
	items.clear();
	for (int i=0;i<itemsCount;i++){
		setConfigValue (mItems[i],"MenuItemPic");
		setConfigValue (mItems[i],"MenuItemDescription");
		setConfigValue (mItems[i],"MenuItemCommand");
		setConfigValue (mItems[i],"MenuItemDbusCall");
		itemType it;
		it.picture.load (workValue["MenuItemPic"]);
		it.description = (workValue["MenuItemDescription"]);
		it.command = (workValue["MenuItemCommand"]);
		it.dbusCall = (workValue["MenuItemDbusCall"]);
		items.append(it);
		}

	shadowPic = new QPixmap (workValue["MenuItemShadowPic"]);
	overPic = new QPixmap (workValue["MenuItemOverPic"]);
	backgroundPic = new QPixmap (workValue["MenuBackgroundPic"]);
	foregroundPic = new QPixmap (workValue["MenuForegroundPic"]);

	setAttribute (Qt::WA_TranslucentBackground);
	resize (mSize, mSize);
	move (calcHPosition (workValue["MenuLeft"], mSize, 0),calcVPosition (workValue["MenuTop"], mSize, 0));
	setWindowOpacity (workValue["MenuOpacity"].toDouble());

	radius=(mSize - (itemPicSize+itemPicSize/2)) / 2 - offset;
	cAngle=90;
	if (itemsCount < 3) cAngle=180;
	
	scene->clear();
	scene->setSceneRect(0,0,mSize,mSize);
	view->resize (mSize,mSize);

	if (!backgroundPic->isNull()){
		bgPic->resize(mSize,mSize);
		bgPic->setPixmap (*backgroundPic);
	}
	else bgPic->clear();
	if (!shadowPic->isNull()){
		slPic->resize (itemPicSize + itemPicSize/2,itemPicSize + itemPicSize/2);
		slPic->setPixmap(*shadowPic);
		}
	else slPic->clear();

	for (int picNumber=0;picNumber < itemsCount;picNumber++){
		double angle = ((360 / itemsCount) * picNumber - cAngle) * M_PI / 180;
 		int cx=radius * cos (angle) + radius + offset;
 		int cy=radius * sin (angle) + radius + offset;
		
 		if (picNumber == current){
 			if (!shadowPic->isNull()) slPic->move(cx,cy);
 			if (!overPic->isNull()) olPic->move(cx,cy);
 		}

		QGraphicsItem *iPic = scene->addPixmap(items[picNumber].picture.scaled(itemPicSize,itemPicSize,Qt::IgnoreAspectRatio,Qt::SmoothTransformation));
		iPic->setPos (cx+itemPicSize/4,cy+itemPicSize/4);
		}
	if (!overPic->isNull()){
		olPic->resize (itemPicSize + itemPicSize/2,itemPicSize + itemPicSize/2);
		olPic->setPixmap(*overPic);
		}
	else olPic->clear();
	if (kCenter != 0){
		clPic->resize (itemPicSize*kCenter,itemPicSize*kCenter);
		clPic->setPixmap(items[current].picture);
		clPic->move((width()-itemPicSize*kCenter)/2,(width()-itemPicSize*kCenter)/2);
		}
	if (!foregroundPic->isNull()){
		fgPic->resize (mSize,mSize);
		fgPic->setPixmap (*foregroundPic);
	}
	else fgPic->clear();

	view->setScene(scene);
	view->show();
	changeMenu();
}

void menuObject::changeMenu (){
	double angle = ((360 / itemsCount) * current - cAngle) * M_PI / 180;
	int cx=radius * cos (angle) + radius + offset;
	int cy=radius * sin (angle) + radius + offset;
	
	QPropertyAnimation* slAnim = new QPropertyAnimation(slPic, "geometry");
	slAnim->setEasingCurve(animTypeMove);
	slAnim->setStartValue(QRect (slPic->x(),  slPic->y(), slPic->width(), slPic->height()));
	slAnim->setEndValue(QRect (cx, cy,  slPic->width(), slPic->height()));
	slAnim->setDuration(animTime);
	slAnim->start();

	QPropertyAnimation* olAnim = new QPropertyAnimation(olPic, "geometry");
	olAnim->setEasingCurve(animTypeMove);
	olAnim->setStartValue(QRect (olPic->x(),  olPic->y(), olPic->width(), olPic->height()));
	olAnim->setEndValue(QRect (cx, cy,  olPic->width(), olPic->height()));
	olAnim->setDuration(animTime);
	olAnim->start();

	if (kCenter != 0){
		clPic->setPixmap(items[current].picture.scaled(itemPicSize*kCenter,itemPicSize*kCenter,Qt::IgnoreAspectRatio,Qt::SmoothTransformation));
		QPropertyAnimation* clAnim = new QPropertyAnimation(clPic, "geometry");
		clAnim->setEasingCurve(animTypeCenter);
		clAnim->setStartValue(QRect (mSize/2, mSize/2,  0, 0));
		clAnim->setEndValue(QRect ((width()-itemPicSize*kCenter)/2,(width()-itemPicSize*kCenter)/2, itemPicSize*kCenter,itemPicSize*kCenter));
		clAnim->setDuration(animTime*4);
		clAnim->start();
	}

	if (!isVisible()){
		QPropertyAnimation* mainAnim = new QPropertyAnimation(this, "windowOpacity");
		mainAnim->setEasingCurve(animTypeFade);
		mainAnim->setStartValue(0);
		mainAnim->setEndValue(workValue["MenuOpacity"].toDouble());
		mainAnim->setDuration(animTime);
		mainAnim->start();
		
		show();
		}
	emit printText (workValue["MenuTextProfile"], items[current].description, msTimeOff);
	timeOff->start (msTimeOff);
	isSelected=false;
}

void menuObject::hideMenu(){
	QTimer::singleShot(animTime, this, SLOT(contMenu()));
	if (isVisible()){
		QPropertyAnimation* mainAnim = new QPropertyAnimation(this, "windowOpacity");
		mainAnim->setEasingCurve(animTypeFade);
		mainAnim->setStartValue(workValue["MenuOpacity"].toDouble());
		mainAnim->setEndValue(0);
		mainAnim->setDuration(animTime);
		mainAnim->start();
	}
}

void menuObject::contMenu(){
	hide();
	if (isSelected){
		timeOff->stop();
		emit hideText();
		if ((! items[current].command.isEmpty()) && (! items[current].command.isNull())){
if (debug) qDebug()<<"---------------- Menu: Run command: "<<items[current].command;
			itemProc = new QProcess ();
			itemProc->startDetached(items[current].command);
			}
		if ((! items[current].dbusCall.isEmpty()) && (! items[current].dbusCall.isNull())){
if (debug) qDebug()<<"---------------- Menu: Call DBUS: "<<items[current].dbusCall;
			dbusCall (items[current].dbusCall);
			}
	}
}

void menuObject::next (QString profile){
	if (!isVisible()){
		showMenu(profile);
	}
	else{
 		if (current < itemsCount-1) current++;
 		else current=0;
 		saveParameterToConfig (cProfile,"MenuCurrentItem",current);
		changeMenu();
	}
}

void menuObject::prev (QString profile){
	if (!isVisible()){
		showMenu(profile);
	}
	else{
		if (current > 0) current--;
		else current=itemsCount-1;
		saveParameterToConfig (cProfile,"MenuCurrentItem",current);
		changeMenu ();
	}
}

void menuObject::select (){
	if (isVisible()){
		isSelected=true;
		hideMenu();
	}
}
