#include "roundIndicator.h"

roundIndicator::roundIndicator( QLabel * parent) : QLabel (parent, Qt::X11BypassWindowManagerHint){
	dbusServiceRegister ("RoundIndicator", this);
	time = new QTimer (this);
	connect (time, SIGNAL(timeout()), this, SLOT(hideIndicator()));
	animationTimer = new QTimer (this);
	connect (animationTimer, SIGNAL(timeout()), this, SLOT(nextStepAnimation()));

	picture.load (workValue["RoundIndicatorPicture"]);
	currentValue=50;
	workValuesToCurrent();
}

roundIndicator::~roundIndicator(){
}

// При перемещении мыши
void roundIndicator::mouseMoveEvent (QMouseEvent *e){
	move (e->globalPos()-clickPos);
	QDBusInterface m ("org.kochkin.okindd-config", "/Configure");
	m.call ("RoundIndicatorPositionChanged", x(), y());
}

// При нажатии ЛКМ
void roundIndicator::mousePressEvent (QMouseEvent *e){
	clickPos = e->pos();
	setCursor (Qt::SizeAllCursor);
	QDBusInterface m ("org.kochkin.okindd-config", "/Configure");
	m.call ("RoundIndicatorPressed");
}

// При освобождении ЛКМ
void roundIndicator::mouseReleaseEvent (QMouseEvent *){
	setCursor (Qt::ArrowCursor);
	QDBusInterface m ("org.kochkin.okindd-config", "/Configure");
	m.call ("RoundIndicatorRelease");
}

// Установка позиции индикатора
void roundIndicator::setPosition(QString hPosition, QString vPosition){
	posHor=calcHPosition (hPosition, fullDiameter, workValue["RoundIndicatorOffset"].toInt());
	posVert=calcVPosition (vPosition, fullDiameter, workValue["RoundIndicatorOffset"].toInt());
	workValue["RoundIndicatorLeft"]=posHor;
	workValue["RoundIndicatorTop"]=posVert;
}

// Получение диаметра внутренней окружности
int roundIndicator::getCenterDiameter(QString CenterDiameter){
	int cd=0;
	if (CenterDiameter.toUpper() == "DESCRIBED") cd=sqrt (pow (picture.width(),2)*2);
		else
			if (CenterDiameter.toUpper() == "INSCRIBED") cd=picture.width();
				else cd=CenterDiameter.toInt();
	return cd;
}

// Установка параметров шрифта
void roundIndicator::setDigitsFont(QString name, bool bold, bool italic){
	digitsFont.setFamily (name);
	digitsFont.setBold(bold);
	digitsFont.setItalic(italic);
}

// Скрыть индикатор
void roundIndicator::hideIndicator(){
	if (workValue["RoundIndicatorTimer"].toInt() != 0) hide();
	time->stop();
}

// Вызов посредством DBUS
void roundIndicator::Indicator (QString File, QString Left, QString Top, QString Diameter, QString Angle, int value, int Time){
	currentValue=getValueFromDBUS(QVariant(value)).toInt();
	offset=16;
	if (!File.isEmpty()) picture.load (File);
	diameter=Diameter.toInt();
	fullDiameter=diameter+offset*2;
	setPosition (Left, Top);
	centerDiameter=getCenterDiameter ("DESCRIBED");
	setAngle (Angle.toInt());
	valueMax=100;
	valueMin=0;
	colorULB=strToColor ("50,100,120,255");
	colorBRB=strToColor ("0,0,0,255");
	colorULI=strToColor ("255,255,255,255");
	colorBRI=strToColor ("0,20,30,255");
	colorULC=strToColor ("0,0,0,255");
	colorBRC=strToColor ("0,80,90,255");
	digitsEnabled=true;
	setDigitsFont ("Liberation sans", true, false);
	colorDigits=strToColor ("255,255,255,50");
	colorDigitsShadow=strToColor ("0,0,0,100");
	opacity=0.8;

	time->stop();
	if (time != 0) time->start (Time);
	showIndicator();
}

// Вызов из функции display
void roundIndicator::showCurrent (int value){
	prevValue=currentValue;
	currentValue=getValueFromDBUS(QVariant (value)).toInt();
	workValuesToCurrent ();

	time->stop();
	if (time != 0) time->start (workValue["RoundIndicatorTimer"].toInt());

	showIndicator();
}

// Копирование рабочих параметров в текущие
void roundIndicator::workValuesToCurrent (){
	offset=16;
	setDiameter (workValue["RoundIndicatorDiameter"].toInt());
	fullDiameter=diameter+offset*2;
	setPosition (workValue["RoundIndicatorLeft"], workValue["RoundIndicatorTop"]);
	centerDiameter=getCenterDiameter (workValue["RoundIndicatorCenterDiameter"]);
	setAngle (workValue["RoundIndicatorAngle"].toInt());
	valueMax=workValue["RoundIndicatorValueMax"].toInt();
	valueMin=workValue["RoundIndicatorValueMin"].toInt();
	colorULB=strToColor (workValue["RoundIndicatorColorULB"]);
	colorBRB=strToColor (workValue["RoundIndicatorColorBRB"]);
	colorULI=strToColor (workValue["RoundIndicatorColorULI"]);
	colorBRI=strToColor (workValue["RoundIndicatorColorBRI"]);
	colorULC=strToColor (workValue["RoundIndicatorColorULC"]);
	colorBRC=strToColor (workValue["RoundIndicatorColorBRC"]);
	digitsEnabled=strToBool (workValue["RoundIndicatorDigitsEnabled"]);
	setDigitsFont (workValue["RoundIndicatorDigitsFont"], strToBool (workValue["RoundIndicatorDigitsFontBold"]), strToBool (workValue["RoundIndicatorDigitsFontItalic"]));
	colorDigits=strToColor (workValue["RoundIndicatorDigitsColor"]);
	colorDigitsShadow=strToColor (workValue["RoundIndicatorDigitsColorShadow"]);
	setOpacity (workValue["RoundIndicatorOpacity"]);
}

// Установка диаметра внешней окружности
 void roundIndicator::setDiameter (int Diameter){
 	QDesktopWidget desktop;
 	if (Diameter<desktop.height()-offset*2) diameter=Diameter;
 		else diameter=desktop.height()-offset*2;
 	fullDiameter=diameter+offset*2;
 	workValue["RoundIndicatorDiameter"]=diameter;
 }

// Установка угла сектора
void roundIndicator::setAngle (int Angle){
	if (Angle<0) angle=0;
		else if (Angle>360) angle=360;
			else angle=Angle;
}

// Установка прозрачности
void roundIndicator::setOpacity (QString Opacity){
	if (Opacity.toFloat()<0) opacity=0;
		else if (Opacity.toFloat()>1) opacity=1;
			else opacity=Opacity.toFloat();
}

QVariant roundIndicator::getValueFromDBUS(QVariant altValue){
	if (workValue["RoundIndicatorValueFromDBUS"] != ""){
		QVariant s = dbusCall (workValue["RoundIndicatorValueFromDBUS"]);
		return s;
		}
	else return altValue;
}

void roundIndicator::showIndicator(){
	if (!QX11Info::isCompositingManagerRunning()) setAttribute (Qt::WA_PaintOnScreen);
	setAttribute (Qt::WA_TranslucentBackground);

	angleLength=angle;
	valueRange=valueMax-valueMin;
	angleBegin=(360-angleLength)/2-90;
	anglePos=(currentValue-valueMin)*angleLength/valueRange;

	prevAngle=(prevValue-valueMin)*angleLength/valueRange;
	animationFPS=30;
	animationX=0.0;
	animationPass=1;
	animationTimer->start(1000/animationFPS);

	resize (fullDiameter, fullDiameter);
	move (posHor,posVert);
	setWindowOpacity (opacity);

	show();
}

void roundIndicator::nextStepAnimation(){
	if (animationPass == 1){
		animationX=animationX+3;
		if (animationX < animationFPS){
			valueOut=prevAngle+(anglePos-prevAngle)/animationFPS*animationX;
			update();
			}
		else{
			animationPass=2;
			animationX=0.0;
			}
		}

	if (animationPass == 2){
		animationX=animationX+0.5;
		if (animationX < animationFPS){
			valueOut=anglePos+(qSin(animationX/2)*(animationFPS-animationX)/animationFPS)*10;
			update();
			}
		else animationTimer->stop();
		}
}

void roundIndicator::paintEvent (QPaintEvent *){
	QPainter *p = new QPainter (this);
	p->setRenderHints (QPainter::Antialiasing|QPainter::TextAntialiasing|QPainter::SmoothPixmapTransform|QPainter::HighQualityAntialiasing);
	p->setBackground (Qt::transparent);
	p->setPen (Qt::NoPen);

	if (QX11Info::isCompositingManagerRunning()) p->drawPixmap (0,0,diameter+offset*2,diameter+offset*2,shadow);
	QRadialGradient gr1 (diameter/3,diameter/3, diameter/1.4, diameter/2-diameter/6,diameter/2-diameter/6);
	gr1.setColorAt (0,colorULB);
	gr1.setColorAt (1,colorBRB);
	p->setBrush (gr1);
	p->drawPie (offset,offset,diameter,diameter, angleBegin*16, angleLength*16);
// Индикатор
	p->setCompositionMode (QPainter::CompositionMode_SourceOver);
	QRadialGradient gr3 (diameter/3,diameter/3, diameter/1.4, diameter/2-diameter/6,diameter/2-diameter/6);
	gr3.setColorAt (0,colorULI);
	gr3.setColorAt (1,colorBRI);
  p->setBrush (gr3);
	p->drawPie (offset,offset,diameter,diameter, (angleLength-valueOut+angleBegin)*16, valueOut*16);
// Центр
	p->setCompositionMode (QPainter::CompositionMode_SourceOver);
	QRadialGradient gr2 (diameter/2-centerDiameter/3, fullDiameter/2-centerDiameter/3, centerDiameter/.9, fullDiameter/2-centerDiameter/6, fullDiameter/2-centerDiameter/6);
	gr2.setColorAt (0,colorULC);
	gr2.setColorAt (1,colorBRC);
	p->setBrush (gr2);
	p->drawEllipse ((diameter-centerDiameter)/2+offset, (diameter-centerDiameter)/2+offset, centerDiameter, centerDiameter);

	if (digitsEnabled){
		QString sValue;
		sValue.setNum (currentValue);
		int fontHeight=(diameter-(centerDiameter+10))/2.6;
		digitsFont.setPixelSize (fontHeight);
		p->setFont (digitsFont);
		p->setPen (colorDigitsShadow);
		p->drawText ((fullDiameter-fullDiameter/1.2)/2+1, ((fontHeight*2.5)-fontHeight*2)/2+1+offset+sValue.size(), fullDiameter/1.2, fontHeight, Qt::AlignCenter,sValue,0);
		p->setPen (colorDigits);
		p->drawText ((fullDiameter-fullDiameter/1.2)/2, ((fontHeight*2.5)-fontHeight*2)/2+offset+sValue.size(), fullDiameter/1.2, fontHeight, Qt::AlignCenter,sValue,0);
		}
	p->setCompositionMode (QPainter::CompositionMode_SourceOver);
	p->drawPixmap ((diameter-picture.width())/2+offset,(diameter-picture.height())/2+offset,picture);
	p->end();
	
	QImage im = QImage (fullDiameter, fullDiameter, QImage::Format_ARGB32_Premultiplied);
	im.fill(0);
	QPainter mp;
	mp.begin (&im);
	mp.setBrush (gr1);
	mp.drawPie (offset+1,offset+1,diameter-2,diameter-2, angleBegin*16+1, angleLength*16);
	mp.drawEllipse ((diameter-centerDiameter)/2+offset, (diameter-centerDiameter)/2+offset, centerDiameter, centerDiameter);
	mp.end();
	if (!QX11Info::isCompositingManagerRunning()) setMask (QPixmap::fromImage(im).mask());
}
