#include "text.h"

textObject::textObject(QLabel * parent):QLabel(parent, Qt::X11BypassWindowManagerHint){
	dbusServiceRegister ("Text", this);
	messLabel = new QLabel(this,Qt::X11BypassWindowManagerHint);
	shadow = new QPixmap;
	timer = new QTimer (this);
	connect (timer, SIGNAL (timeout()), this, SLOT (hideText()));
}

textObject::~textObject(){
}

// Вывод сообщения message в соответствии с параметрами профиля profile и на время time
// message поддерживает html/css форматирование
void textObject::printText (QString profile, QString message, int time){
	offset=16;
	setConfigValue (profile,"TextBoxLeft");
	setConfigValue (profile,"TextBoxTop");
	setConfigValue (profile,"TextBoxStyle");
	setConfigValue (profile,"TextBoxOpacity");
	setConfigValue (profile,"TextBoxBorderSize");
	setConfigValue (profile,"TextBoxOffset");
	setConfigValue (profile,"TextBoxTransparent");
	setConfigValue (profile,"TextBoxShadow");
	if (QX11Info::isCompositingManagerRunning()) {
		setAttribute (Qt::WA_TranslucentBackground, true);
		messLabel->setAttribute (Qt::WA_TranslucentBackground, strToBool (workValue["TextBoxTransparent"]));
		messLabel->setAttribute (Qt::WA_NoSystemBackground, strToBool (workValue["TextBoxTransparent"]));
		if (!strToBool (workValue["TextBoxTransparent"]) && workValue["TextBoxShadow"] != "") shadow->load (workValue["TextBoxShadow"]);
		}
	messLabel->setAlignment (Qt::AlignCenter);
	messLabel->setStyleSheet (workValue["TextBoxStyle"]);
	messLabel->setText (message);
	messLabel->adjustSize();
	messLabel->resize (messLabel->width()+workValue["TextBoxBorderSize"].toInt(), messLabel->height()+workValue["TextBoxBorderSize"].toInt());
	if (QX11Info::isCompositingManagerRunning()) {
		messLabel->resize (messLabel->width()+workValue["TextBoxBorderSize"].toInt(), messLabel->height()+workValue["TextBoxBorderSize"].toInt());
		resize (messLabel->width()+offset*2, messLabel->height()+offset*2);
		messLabel->move (offset,offset);
		}
	else {
		messLabel->resize (messLabel->width()+workValue["TextBoxBorderSize"].toInt(), messLabel->height()+workValue["TextBoxBorderSize"].toInt());
		resize (messLabel->width(), messLabel->height());
		messLabel->move (0,0);
		}
	move (calcHPosition (workValue["TextBoxLeft"], width(), workValue["TextBoxOffset"].toInt()),calcVPosition (workValue["TextBoxTop"], height(), workValue["TextBoxOffset"].toInt()));
	timer->stop();	
	timer->start (time);
	setWindowOpacity (workValue["TextBoxOpacity"].toFloat());

	show();
// Непонятное поведение. Иногда (в 5%-10% показов), сообщение не прорисовывается полностью или вообще не показывается. После показа сделана перерисовка.
	QTimer::singleShot(50,this,SLOT (repaint()));
}

void textObject::paintEvent (QPaintEvent *){
	if (QX11Info::isCompositingManagerRunning() && !strToBool (workValue["TextBoxTransparent"])){
		QPainter *p = new QPainter (this);
		p->setRenderHints (QPainter::Antialiasing|QPainter::TextAntialiasing|QPainter::SmoothPixmapTransform|QPainter::HighQualityAntialiasing);
		p->setBackground (Qt::transparent);
		p->setPen (Qt::NoPen);
		if (workValue["TextBoxShadow"] != "") p->drawPixmap (0,0,width(),height(),*shadow);
		p->end();
		}
}

void textObject::hideText(){
	timer->stop();
	hide();
}

// Вывод сообщения messWithDBUS в соответствии с параметрами профиля profile и на время time
// messWithDBUS поддерживает html/css форматирование
// messWithDBUS может включать вызовы DBUS, обрамлённые тегами {[...]}
void textObject::printTextWithDBUS (QString profile, QString messWithDBUS, int time){
	QStringList allMess=messWithDBUS.split (QRegExp ("\\{|\\}"));
	int count=allMess.size();
	for (int i=0;i<count;i++){
		if (allMess[i][0] == '[' && allMess[i][allMess[i].size()-1] == ']'){
			allMess[i]=allMess[i].right (allMess[i].size()-1);
			allMess[i]=allMess[i].left (allMess[i].size()-1);
			allMess[i]=dbusCall (allMess[i]).toString();
			}
		}
	printText (profile, allMess.join(""), time);
}

